"use client";

import * as React from "react";
import {
  Sidebar,
  SidebarContent,
  SidebarGroup,
  SidebarGroupContent,
  SidebarGroupLabel,
  SidebarMenu,
  SidebarMenuButton,
  SidebarMenuItem,
  SidebarMenuSub,
  SidebarRail,
} from "@/components/ui/sidebar";
import { GitEntry } from "gitea-js";
import {
  Collapsible,
  CollapsibleContent,
  CollapsibleTrigger,
} from "@/components/ui/collapsible";
import { useSession } from "next-auth/react";
import { retrieveTreeStructure } from "@/actions";
import { COriginal, JavaOriginal } from "devicons-react";
import { ChevronRight, File, Folder, FolderOpen } from "lucide-react";

interface FileTree {
  name: string;
  type: "blob" | "tree";
  children?: { [key: string]: FileTree };
}

export function buildFileTree(tree: GitEntry[]): FileTree {
  const root: FileTree = { name: "", type: "tree", children: {} };

  tree.forEach((entry: GitEntry) => {
    if (!entry.path) return;

    const pathParts = entry.path.split("/");
    let currentLevel = root;

    pathParts.forEach((part: string) => {
      if (!currentLevel.children) {
        currentLevel.children = {};
      }

      if (!currentLevel.children[part]) {
        currentLevel.children[part] = {
          name: part,
          type: entry.type as "blob" | "tree",
          children: {},
        };
      }

      currentLevel = currentLevel.children[part];
    });
  });

  if (root.children) {
    const sortedChildren = Object.values(root.children).sort((a, b) => {
      if (a.type === "tree" && b.type === "blob") return -1;
      if (a.type === "blob" && b.type === "tree") return 1;
      return a.name.localeCompare(b.name);
    });

    root.children = sortedChildren.reduce((acc, child) => {
      acc[child.name] = child;
      return acc;
    }, {} as { [key: string]: FileTree });
  }

  return root;
}

export function PlaygroundSidebar({
  ...props
}: React.ComponentProps<typeof Sidebar>) {
  const { data: session } = useSession();
  const [fileTree, setFileTree] = React.useState<FileTree>({
    name: "",
    type: "tree",
    children: {},
  });

  const username = session?.user?.name ?? "";

  React.useEffect(() => {
    async function fetchFileTree() {
      if (username) {
        const tree: GitEntry[] = await retrieveTreeStructure(
          username,
          "playground",
          "main"
        );
        const newFileTree = buildFileTree(tree);
        setFileTree(newFileTree);
      }
    }
    fetchFileTree();
  }, [username]);

  return (
    <Sidebar {...props}>
      <SidebarContent>
        <SidebarGroup>
          <SidebarGroupLabel>Files</SidebarGroupLabel>
          <SidebarGroupContent>
            <SidebarMenu>
              {fileTree.children &&
                Object.values(fileTree.children).map((item, index) => (
                  <Tree key={index} item={item} />
                ))}
            </SidebarMenu>
          </SidebarGroupContent>
        </SidebarGroup>
      </SidebarContent>
      <SidebarRail />
    </Sidebar>
  );
}

function Tree({ item }: { item: FileTree }) {
  const { name, type, children } = item;
  const fileExtension = name.split(".").pop();

  let fileIcon = <File />;

  if (fileExtension === "c") {
    fileIcon = <COriginal />;
  } else if (fileExtension === "java") {
    fileIcon = <JavaOriginal />;
  }

  const [isOpen, setIsOpen] = React.useState(false);

  if (type === "blob") {
    return (
      <SidebarMenuButton className="data-[active=true]:bg-transparent">
        {fileIcon}
        {name}
      </SidebarMenuButton>
    );
  }

  return (
    <SidebarMenuItem>
      <Collapsible
        open={isOpen}
        onOpenChange={setIsOpen}
        className="group/collapsible [&[data-state=open]>button>svg:first-child]:rotate-90"
      >
        <CollapsibleTrigger asChild>
          <SidebarMenuButton>
            <ChevronRight className="transition-transform" />
            {isOpen ? <FolderOpen /> : <Folder />}
            {name}
          </SidebarMenuButton>
        </CollapsibleTrigger>
        <CollapsibleContent>
          <SidebarMenuSub>
            {children &&
              Object.values(children).map((subItem, index) => (
                <Tree key={index} item={subItem} />
              ))}
          </SidebarMenuSub>
        </CollapsibleContent>
      </Collapsible>
    </SidebarMenuItem>
  );
}
